package fr.velossity.sample.device.impl;

import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import org.osgi.framework.BundleContext;

import fr.velossity.sample.device.Device;
import fr.velossity.sample.device.Measurement;
import fr.velossity.sample.device.eventing.MeasurementListener;
import fr.velossity.sample.device.eventing.MeasurementProducer;

/**
 * A mock device implementation.
 * @author C. Saint-Marcel
 * 
 */
public class MockDeviceImpl implements Device, MeasurementProducer, Runnable {

	/**
	 * The device identifier.
	 */
	private String identifier;


	/**
	 * The bundle context.
	 */
	private BundleContext myContext;

	/**
	 * A thread to generate random measures each 10 seconds.
	 */
	private Thread measureGenerator;

	/**
	 * The last measurement randomly generated.
	 */
	private Measurement lastMeasurement;
	
	/**
	 * The internal list of listeners maintained by the device itself. It will be
	 * duplicated for each device instanciation.
	 */
	private List listeners;

	/**
	 * Default constructor.
	 */
	public MockDeviceImpl(String identifier, BundleContext context) {
		super();
		myContext = context;
		this.identifier = identifier;
	}

	/**
	 * @see Device#getIdentifier()
	 */
	public String getIdentifier() {
		return identifier;
	}

	/**
	 * @see Device#getType()
	 */
	public String getType() {
		return "Mock";
	}

	/**
	 * Mock device is always functional.
	 * @see Device#getState()
	 */
	public String getState() {
		return DEVICE_STATE_FUNCTIONAL;
	}

	/**
	 * Activates the mock device by registering it as a {@link MeasurementProducer}
	 * and starting the generator thread.
	 */
	public synchronized void activate() {
		listeners = new ArrayList();
		if(measureGenerator == null) {
			measureGenerator = new Thread(this);
			measureGenerator.start();
		}
		// REGISTERS the MeasurementProducer
		Dictionary properties = new Hashtable();
		properties.put("identifier", identifier);
		myContext.registerService(MeasurementProducer.class.getName(), this, properties );
	}

	/**
	 * Stops the generator.
	 */
	public synchronized void deactivate() {
		measureGenerator = null;
	}

	public void run() {
		Thread mg = measureGenerator;
		while (measureGenerator == mg) {
			// Regenerate last measurement
			lastMeasurement = new Measurement(MockDeviceImpl.this, new Random().nextFloat()* 1000, "Watt", System.currentTimeMillis());
			
			// NOTIFY each listener implementing MeasurementListener interface
			synchronized (listeners) {
				for (Iterator iterator = listeners.iterator(); iterator
						.hasNext();) {
					MeasurementListener aListener = (MeasurementListener) iterator.next();
					aListener.newMeasure(lastMeasurement);
				}
			}
			try {
				synchronized (this) {
					wait(10000);	
				}				
			} catch (InterruptedException e) {
				e.printStackTrace();
			}

		}

	}

	/**
	 * @see MeasurementProducer#addListener(MeasurementListener)
	 */
	public void addListener(MeasurementListener consumer) {
		synchronized (listeners) {
			listeners.add(consumer);			
		}
	}

	/**
	 * @see MeasurementProducer#removeListener(MeasurementListener)
	 */
	public void removeListener(MeasurementListener consumer) {
		synchronized (listeners) {
			listeners.remove(consumer);			
		}
	}

}
